using System;
using System.Collections.Generic;
using System.Text;
using RJH.CommandLineHelper;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Proxies;
using SWConfigDataClientLib.Proxies.Users;
using System.Runtime.InteropServices;

namespace $safeprojectname$
{
    public class Program 
    {
        #region Commandline parameters

        private string _cmdLineservername;

        [CommandLineSwitch("s", "IpPbxServer to use (localhost is used when not specified)")]
        [ComVisible(false)]
        public string CmdLineServerName
        {
            get { return _cmdLineservername; }
            set { _cmdLineservername = value; }
        }

        private string _Username;
        [CommandLineSwitch("u", "<username>", "user name to login to SwyxServer. Uses current windows account if not specified")]
        [ComVisible(false)]
        public string CmdLineUsername
        {
            get { return _Username; }
            set { _Username = value; }
        }

        private string _password;

        [CommandLineSwitch("p", "<password>", "password for login to SwyxServer. Required when username is specified")]
        [ComVisible(false)]
        public string CmdLinePassword
        {
            get { return _password; }
            set { _password = value; }
        }

        private bool _showHelp;

        [CommandLineSwitch("?", "Show help")]
        [ComVisible(false)]
        public bool CmdLineShowHelp
        {
            get { return _showHelp; }
            set { _showHelp = value; }
        }

        #endregion

        #region CDS
        LibManager _libManager;

        ValueList _usrIds;
        bool _bConnected;
        #endregion

        static int Main(string[] args)
        {
            Program p = new Program();
            return p.Run();
        }

        private int Run()
        {
            int result = 0;
            try
            {
                Parser parser = new Parser(System.Environment.CommandLine, this);
                parser.Parse();

                // TODO: display help when required parameters are missing
                if (CmdLineShowHelp)
                {
                    System.Reflection.Assembly exe = System.Reflection.Assembly.GetExecutingAssembly();
                    parser.ShowHelp(exe.GetModules()[0].Name);
                    return 0;
                }

                // TODO: react on commandline parameter
            }
            catch (System.Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            return result;
        }

        private int GetUserID(string username)
        {
            int UserID = 0;
            if (username != null && username != "")
            {
                foreach (ValueItem v in _usrIds)
                {
                    if (string.Compare(v.Text, username, true) == 0)
                    {
                        UserID = v.KeyInt;
                        break;
                    }
                }
                if (UserID == 0)
                {
                    Console.WriteLine("User '{0}' is unknown.", username);
                }
            }
            return UserID;
        }

        public bool DisconnectFromCDS()
        {
            if (!_bConnected)
                return false;

            if (_libManager != null)
            {
                _libManager = null;
            }
            return true;
        }


        private bool ConnectToCDS()
        {
            if (!_bConnected)
                return ConnectToCDS(CmdLineServerName, CmdLineUsername, CmdLinePassword);
            return true;
        }

        public bool ConnectToCDS(string servername, string username, string password)
        {
            if (_bConnected)
                return true;

            try
            {
                if (_libManager == null)
                    _libManager = new LibManager();

                if (username != null && username != "")
                {
                    _libManager.Username = "ippbx:" + username;
                    _libManager.Password = password;
                    _libManager.AuthenticationMode = SProxyObject.AuthenticationMode.UsernamePassword;
                }
                else
                {
                    if (servername == null || servername == "")
                        _libManager.AuthenticationMode = SProxyObject.AuthenticationMode.TrustedLocal;
                    else

                        _libManager.AuthenticationMode = SProxyObject.AuthenticationMode.Trusted;
                }

                _libManager.BaseProtocol = SProxyObject.BaseProtocol.TCP;

                if (servername != null)
                    _libManager.WSBaseUrl = servername;
                else
                    _libManager.WSBaseUrl = "127.0.0.1";

                // get user list
                UserEnum _usrEnum = _libManager.GetUserEnum();
                ValueListCollection vlc = _usrEnum.getValueLists(SWConfigDataClientLib.Proxies.Users.ValueLists.UserID);
                _usrIds = vlc["UserID"];

                // TODO: add additional CDS intialization here

                
                
                _bConnected = true;
            }
            catch (SWConfigDataSharedLib.Exceptions.UserDbException ue)
            {
                Console.WriteLine(ue.ToString());
                _bConnected = false;
                return false;
            }
            catch (SWConfigDataSharedLib.Exceptions.SBaseException be)
            {
                Console.WriteLine(be.ToString());
                _bConnected = false;
                return false;
            }

            return true;
        }
    }
}
