using System;
using System.Collections.Generic;
using System.Text;
using RJH.CommandLineHelper;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Proxies.Files;
using SWConfigDataClientLib.Proxies;
using SWConfigDataClientLib.Proxies.Users;
using System.IO;
using System.Runtime.InteropServices;

namespace IpPbxFile
{
    [ComVisible(true)]
    public class FileManager : IpPbxFile.IFileManager
    {
        #region Commandline parameters

        private string _cmdLineservername;

        [CommandLineSwitch("s", "IpPbxServer to use (localhost is used when not specified)")]
        [ComVisible(false)]
        public string CmdLineServerName
        {
            get { return _cmdLineservername; }
            set { _cmdLineservername = value; }
        }

        [ComVisible(false)]
        public enum CommandLineFileScope
        {
            User,
            UserDefault,
            Global,
            SystemDefault
        }

        private FileScope _FileScope;

        [CommandLineSwitch("fs", "<User|UserDefault|Global|SystemDefault>", "file scope to upload to (User,UserDefault,Global,SystemDefault)")]
        [ComVisible(false)]
        public string CmdLineFileScope
        {
            get { return Enum.GetName(typeof(FileScope), _FileScope); }
            set { _FileScope = (FileScope)Enum.Parse(typeof(FileScope), value); }
        }

        private bool _listFiles;
        [CommandLineSwitch("l", "list files")]
        [ComVisible(false)]
        public bool CmdLineListFiles
        {
            get { return _listFiles; }
            set { _listFiles = value; }
        }

        private string _fileToUpload;

        [CommandLineSwitch("f", "<filename>", "upload <filename>")]
        [ComVisible(false)]
        public string CmdLineFileToUpload
        {
            get { return _fileToUpload; }
            set { _fileToUpload = value; }
        }

        private PreDefinedFileCategory _fileCategory = PreDefinedFileCategory.MaxValue;

        [CommandLineSwitch("fc", "<file category>", "file category (Announcements,Bitmaps,CallRoutingScripts,HoldMusic,RingTones,SampleAnnouncements,SampleScripts,Skins,SystemAnnouncements)")]
        [ComVisible(false)]
        public string CmdLineFileCategory
        {
            get { return System.Enum.GetName(typeof(PreDefinedFileCategory), _fileCategory); }
            set
            {
                _fileCategory = (PreDefinedFileCategory)Enum.Parse(typeof(PreDefinedFileCategory), value);
            }
        }

        private bool _overwrite;

        [CommandLineSwitch("fo", "overwrite existing file, if any")]
        [ComVisible(false)]
        public bool CmdLineOverWrite
        {
            get { return _overwrite; }
            set { _overwrite = value; }
        }

        private string _filename;

        [CommandLineSwitch("fn", "<filename>", "name of file in database. Optional, file name is used when not specified")]
        [ComVisible(false)]
        public string CmdLineFilename
        {
            get { return _filename; }
            set { _filename = value; }
        }

        private string _fileUsername;

        [CommandLineSwitch("fu", "<username>", "user name of the user to upload the file to. Required for file scope user or user default, ignored otherwise")]
        [ComVisible(false)]
        public string CmdLineFileUsername
        {
            get { return _fileUsername; }
            set { _fileUsername = value; }
        }

        private string _Username;
        [CommandLineSwitch("u", "<username>", "user name to login to SwyxServer. Uses current windows account if not specified")]
        [ComVisible(false)]
        public string CmdLineUsername
        {
            get { return _Username; }
            set { _Username = value; }
        }

        private string _password;

        [CommandLineSwitch("p", "<password>", "password for login to SwyxServer. Required when username is specified")]
        [ComVisible(false)]
        public string CmdLinePassword
        {
            get { return _password; }
            set { _password = value; }
        }

        private string _description;

        [CommandLineSwitch("fd", "<description>", "optional file description")]
        [ComVisible(false)]
        public string CmdLineDescription
        {
            get { return _description; }
            set { _description = value; }
        }

        private bool _privateFile;

        [CommandLineSwitch("fp", "if 1, file is marked as private, only the owner of the file can access it. 1 is the default")]
        [ComVisible(false)]
        public bool CmdLinePrivateFile
        {
            get { return _privateFile; }
            set { _privateFile = value; }
        }

        private string _fileIDToDelete;

        [CommandLineSwitch("fr", "<fileID>", "file id of file to remove")]
        [ComVisible(false)]
        public string CmdLineFileIDToDelete
        {
            get { return _fileIDToDelete; }
            set { _fileIDToDelete = value; }
        }

        private bool _showHelp;

        [CommandLineSwitch("?", "Show help")]
        [ComVisible(false)]
        public bool CmdLineShowHelp
        {
            get { return _showHelp; }
            set { _showHelp = value; }
        }

        #endregion

        #region CDS
        LibManager _libManager;
        FilesFacade _fileFacade;
        ValueList _usrIds;
        bool _bConnected;
        #endregion


        static int Main(string[] args)
        {
            FileManager p = new FileManager();
            return p.Run();
        }

        private int Run()
        {
            int result = 0;
            try
            {
                Parser parser = new Parser(System.Environment.CommandLine, this);
                parser.Parse();

                if (CmdLineShowHelp)
                {
                    System.Reflection.Assembly exe = System.Reflection.Assembly.GetExecutingAssembly();
                    parser.ShowHelp(exe.GetModules()[0].Name + "\n\nUpload file to SwyxWare database");
                    return 0;
                }

                if (result == 0 && !CmdLineListFiles && (CmdLineFileIDToDelete == null || CmdLineFileIDToDelete == "") && (CmdLineFileToUpload == null || CmdLineFileToUpload == ""))
                {
                    Console.WriteLine("No file specified");
                    result = 1;
                }

                if (result == 0 && CmdLineListFiles)
                {
                    CDSFileInfo[] fi = GetFileList(CmdLineFilename, CmdLineFileUsername, new string[] { CmdLineFileScope },
                        new string[] { CmdLineFileCategory });
                    if (fi != null)
                    {
                        Console.WriteLine("FileID,Name,Scope,Category,Description,FileSize,OriginalFileSize");
                        foreach (object o in fi)
                        {
                            CDSFileInfo f = (CDSFileInfo)o;
                            Console.WriteLine("'{0}','{1}',{2},{3},'{4}',{5},{6},{7}",
                                f.FileID, f.FileName, f.Scope, f.Category, f.UserName, f.Description, f.CompressedFileSize, f.FileSize);
                        }
                    }
                    else
                        result = 1;
                }
                else if (result == 0 && CmdLineFileIDToDelete != null)
                {
                    if (!DeleteFile(CmdLineFileIDToDelete))
                        result = 1;
                }
                else if (result == 0)
                {
                    result = UploadFile(CmdLineFileToUpload, CmdLineFilename, CmdLineFileUsername, CmdLineFileScope, CmdLineFileCategory, CmdLinePrivateFile, CmdLineDescription, CmdLineOverWrite);
                }

                DisconnectFromCDS();
            }
            catch (System.Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            return result;
        }

        public bool DeleteFile(string fileID)
        {
            int result = 0;
            // connect to ConfigDataStore
            if (!ConnectToCDS())
            {
                result = 1;
            }

            if (result == 0)
            {
                Guid fileGUID = new Guid(CmdLineFileIDToDelete);
                _fileFacade.DeleteIpPbxFile(fileGUID);
                Console.WriteLine("File deleted");
            }

            return result == 0;
        }


        // need to return an object[] here to make COM happy
        public object[] ListAllFiles()
        {
            CDSFileInfo[] fi = GetFileList("%", "", new string[0], new string[0]);
            object[] o = new object[fi.Length];
            fi.CopyTo(o, 0);
            return o;
        }

        // need to return an object[] here to make COM happy
        public object[] ListFiles(string filename)
        {
            CDSFileInfo[] fi = GetFileList(filename, "", new string[0], new string[0]);
            object[] o = new object[fi.Length];
            fi.CopyTo(o, 0);
            return o;
        }

        public object[] ListFiles(string filename, string username, string scopelist, string categorylist)
        {
            string[] scopes = new string[0];
            if (scopelist!="")
                scopes = scopelist.Split(new char[] { ',' });
            string[] categories = new string[0];
            if (categorylist!="")
                categories = categorylist.Split(new char[] { ',' }); ;
            CDSFileInfo[] fi = GetFileList(filename, username, scopes, categories);
            object[] o = new object[fi.Length];
            fi.CopyTo(o, 0);
            return o;
        }

        [ComVisible(false)]
        private CDSFileInfo[] GetFileList(string filename, string username, string[] scopes,string[] categories)
        {
            CDSFileInfo[] fi = null;

            int result = 0;
            // connect to ConfigDataStore
            if (!ConnectToCDS())
            {
                result = 1;
            }

            // find userid
            int UserID = 0;
            if (username != null && username != "")
            {
                UserID = GetUserID(username);
                if (UserID == 0)
                {
                    Console.WriteLine("User '{0}' is unknown.", CmdLineFileUsername);
                    result = 1;
                }
            }

            // get file list
            if (result == 0)
            {
                if (filename == null)
                    filename = "%";
                FileScope fs = 0;
                if (scopes != null)
                {
                    
                    foreach (string s in scopes)
                    {
                        if (s == null || s == "")
                            continue;
                        switch (s)
                        {
                            case "User":
                                fs |= FileScope.User;
                                break;
                            case "UserDefault":
                                fs |= FileScope.UserDefault;
                                break;
                            case "Global":
                                fs |= FileScope.Global;
                                break;
                            case "SystemDefault":
                                fs |= FileScope.SystemDefault;
                                break;
                            default:
                                Console.WriteLine("Unknown file scope {0}", s);
                                result = 1;
                                break;
                        }
                    }
                }

                if (UserID != 0 && (fs & FileScope.User) == 0 && (fs & FileScope.UserDefault) == 0)
                    fs = FileScope.User | FileScope.UserDefault;
                if (fs == 0)
                    fs = FileScope.ALL;

                PreDefinedFileCategory[] fc;

                if (categories == null || categories.Length == 0 || categories[0] == PreDefinedFileCategory.MaxValue.ToString())
                {
                    fc = new PreDefinedFileCategory[] { PreDefinedFileCategory.Announcements , PreDefinedFileCategory.Bitmaps ,
                        PreDefinedFileCategory.CallRoutingScripts , PreDefinedFileCategory.HoldMusic , PreDefinedFileCategory.RingTones ,
                        PreDefinedFileCategory.SampleAnnouncements , PreDefinedFileCategory.SampleScripts , PreDefinedFileCategory.Skins ,
                        PreDefinedFileCategory.SystemAnnouncements };
                }
                else
                {
                    fc = new PreDefinedFileCategory[categories.Length];
                    int i = 0;
                    foreach (string c in categories)
                    {
                        PreDefinedFileCategory pdc = (PreDefinedFileCategory)Enum.Parse(typeof(PreDefinedFileCategory), c);
                        if (pdc != PreDefinedFileCategory.MaxValue)
                            fc[i++] = pdc;
                    }
                }

                FileListPrimaryCollection flc = _fileFacade.GetFileList(filename, fs, fc, UserID, true);
                fi = new CDSFileInfo[flc.Count];
                int idx = 0;
                foreach (FileListEntry fle in flc)
                {
                    CDSFileInfo f = new CDSFileInfo();
                    f.Category = fle.Category;
                    f.Description = fle.Description;
                    f.FileID = fle.FileID.ToString();
                    f.FileName = fle.Name;
                    f.Private = fle.Private;
                    f.Scope = fle.Scope.ToString();
                    if (fle.UserID != 0)
                    {
                        f.UserName = _usrIds[fle.UserID].Text;
                    }
                    f.FileSize = fle.OriginalFileSize;
                    f.CompressedFileSize = fle.FileSize;
                    fi[idx++] = f;
                }
            }

            return fi;
        }

        public int UploadFile(string filepath, string filename, string username, string scope, string category, bool bPrivateFile, string description, bool bOverwrite)
        {
            int result = 0;

            // check if file exists 
            if (result == 0 && !File.Exists(filepath))
            {
                Console.WriteLine("file '{0}' does not exist.", filepath);
                result = 1;
            }

            // use original file is none specified
            if (result == 0 && (filename == null || filename == ""))
                filename = Path.GetFileName(filepath);

            // connect to ConfigDataStore
            if (result == 0 && !ConnectToCDS())
            {
                result = 1;
            }

            // find userid
            int UserID = GetUserID(username);
            if (UserID == 0)
                result = 1;

            // upload the file
            if (result == 0)
            {
                FileScope fs = (FileScope)Enum.Parse(typeof(FileScope), scope);
                PreDefinedFileCategory fc = (PreDefinedFileCategory)Enum.Parse(typeof(PreDefinedFileCategory), category);
                FileListEntry fle = _fileFacade.AddIpPbxFile(filename, fs, fc, UserID, false, bPrivateFile, description, false, false, "", filepath, bOverwrite);
                Console.WriteLine("File uploaded, fileid={0}", fle.FileID);
            }

            return result;
        }

        private int GetUserID(string username)
        {
            int UserID = 0;
            if (username != null && username != "")
            {
                foreach (ValueItem v in _usrIds)
                {
                    if (string.Compare(v.Text, username, true) == 0)
                    {
                        UserID = v.KeyInt;
                        break;
                    }
                }
                if (UserID == 0)
                {
                    Console.WriteLine("User '{0}' is unknown.", username);
                }
            }
            return UserID;
        }

        public bool DisconnectFromCDS()
        {
            if (!_bConnected)
                return false;

            if (_libManager != null)
            {
                _fileFacade = null;
                _libManager = null;
            }
            return true;
        }


        private bool ConnectToCDS()
        {
            if (!_bConnected)
                return ConnectToCDS(CmdLineServerName, CmdLineUsername, CmdLinePassword);
            return true;
        }

        public bool ConnectToCDS(string servername, string username, string password)
        {
            if (_bConnected)
                return true;

            try
            {
                if (_libManager == null)
                    _libManager = new LibManager();

                if (username != null && username != "")
                {
                    _libManager.Username = "ippbx:" + username;
                    _libManager.Password = password;
                    _libManager.AuthenticationMode = SProxyObject.AuthenticationMode.UsernamePassword;
                }
                else
                {
                    if (servername == null || servername == "")
                        _libManager.AuthenticationMode = SProxyObject.AuthenticationMode.TrustedLocal;
                    else

                        _libManager.AuthenticationMode = SProxyObject.AuthenticationMode.Trusted;
                }

                _libManager.BaseProtocol = SProxyObject.BaseProtocol.TCP;

                if (servername != null)
                    _libManager.WSBaseUrl = servername;
                else
                    _libManager.WSBaseUrl = "127.0.0.1";

                _fileFacade = _libManager.GetFilesFacade();

                UserEnum _usrEnum = _libManager.GetUserEnum();
                ValueListCollection vlc = _usrEnum.getValueLists(SWConfigDataClientLib.Proxies.Users.ValueLists.UserID);
                _usrIds = new ValueList();
                _usrIds.Add(0, "(all)", "");
                foreach (ValueItem v in vlc["UserID"])
                {
                    _usrIds.Add(v.Key, v.Text, "");
                }
                _bConnected = true;
            }
            catch (SWConfigDataSharedLib.Exceptions.UserDbException ue)
            {
                Console.WriteLine(ue.ToString());
                _bConnected = false;
                return false;
            }
            catch (SWConfigDataSharedLib.Exceptions.SBaseException be)
            {
                Console.WriteLine(be.ToString());
                _bConnected = false;
                return false;
            }

            return true;
        }
    }
}
